#! /usr/bin/env python3

from gi.repository import Gdk, Gst
import random

Y901_MIMES        = "video/x-ms-wmv|video/x-flv|video/mp4|video/mpeg|video/x-msvideo"
Y901_SPEED_NUMS   = (0.12, 0.25, 0.33, 0.5, 0.67, 1.0, 1.5, 2.0, 3.0, 4.0, 8.0)
Y901_SPEED_LABELS = ("0.12x", "0.25x", "0.33x", "0.50x", "0.67x", "1.00x", "1.50x", "2.00x", "3.00x", "4.00x", "8.00x")
Y901_SIZE_STRS    = ("win","x1.0","x2.0","x3.0","x0.5","x1.5","x2.5","set1","set2","set3")
Y901_SIZE_KEYS    = (Gdk.KEY_0, Gdk.KEY_1, Gdk.KEY_2, Gdk.KEY_3, Gdk.KEY_4, Gdk.KEY_5, Gdk.KEY_6, Gdk.KEY_7, Gdk.KEY_8, Gdk.KEY_9)
Y901_ASPECT_STRS  = ("default", "4:3", "16:9", "custom")
Y901_MENU_STRS    = ("_Open", "_Preference", "_Quit")
Y901_MENU_KEYS    = (Gdk.KEY_o, Gdk.KEY_p, Gdk.KEY_q)

class CRList():
    """
        Random File List
    """
    def __init__(self):
        self.l = []
        self.m = []

    def get(self):
        for s in self.l:
            if not s in self.m:
                self.m.append(s)
                return s
        self.m[:] = []
        return self.l[0]

    def append(self, text):
        self.l.append(text)

    def clear(self):
        self.l[:] = []
        self.m[:] = []

    def shuffle(self):
        random.shuffle(self.l)

class CPath(object):
    __slots__ = ["dirc", "name"]
    def __init__(self):
        self.dirc = ""
        self.name = ""

class CSize(object):
    __slots__ = ["width", "height"]
    def __init__(self, w=0, h=0):
        self.width = w
        self.height = h

class CPoint(object):
    __slots__ = ["x", "y"]
    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

class CGesture(object):
    __slots__ = ["start", "result", "title", "prevchar", "x", "y"]
    def __init__(self):
        self.start = False
        self.result = ""
        self.title = ""
        self.prevchar = ""
        self.x = 0
        self.y = 0

class CSetting(object):
    __slots__ = [
            "mimes", "position", "init_size_on", "init_size_val", "esc_exit",
            "mem_rep", "mem_aspect", "endressloop", "severity", "aspect_rate",
            "set_size", "size_win_only"]
    def __init__(self):
        self.mimes = Y901_MIMES
        self.position = False
        self.init_size_on = False
        self.init_size_val = 0
        self.esc_exit = False
        self.mem_rep = False
        self.mem_aspect = False
        self.endressloop = False
        self.severity = 25
        self.aspect_rate = [1,1]
        self.set_size = [[320,240],[640,480],[1280,720]]
        self.size_win_only = False

class CSettingWin(object):
    __slots__ = [
            "list_width", "list_show", "volume", "video_size", "repeat",
            "aspect", "repeat_mem", "aspect_mem", "n_audio", "speed", "direction"]
    def __init__(self):
        self.list_width = 100
        self.list_show = True
        self.volume = 100.0
        self.video_size = 0
        self.repeat = 0
        self.aspect = 0
        self.repeat_mem = 0
        self.aspect_mem = 0
        self.n_audio = 0
        self.speed = 5
        self.direction = 1.0

class CSeekInfo(object):
    __slots__ = ["is_seek", "playing", "button"]
    def __init__(self):
        self.is_seek = False
        self.playing = False
        self.button  = 0

class CPlayInfo(object):
    __slots__ = ["uri", "is_playing", "vsize", "position", "duration", "framerate"]
    def __init__(self):
        self.uri = None
        self.is_playing = False
        self.vsize = CSize(1, 1)
        self.position = Gst.CLOCK_TIME_NONE
        self.duration = Gst.CLOCK_TIME_NONE
        self.framerate = 0

class CFullInfo(object):
    __slots__ = ["is_fullscreen", "is_show_list", "toolbox", "width", "height"]
    def __init__(self):
        self.is_fullscreen = False
        self.is_show_list = False
        self.toolbox = None
        self.width = 320
        self.height = 240

