#!/usr/bin/env python3

from gi.repository import Gtk

class CLabel(Gtk.Label):
    """
        Markup on and left align label
    """
    def __init__(self, s=None):
        Gtk.Label.__init__(self, s)
        self.set_use_markup(True)
        self.set_alignment(0, 0)


class SettingDialog(Gtk.Dialog):
    """
        Setting Dialog
    """
    def __init__(self, parent, setting):
        Gtk.Dialog.__init__(
                self,
                "Y901x の設定", 
                parent,
                Gtk.DialogFlags.MODAL,
                (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT, Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT)
            )
        # Create GtkNotebook and tab label
        note = Gtk.Notebook()
        page = []
        for s in ("全般", "表示", "リスト"):
            v = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
            v.set_border_width(10)
            page.append(v)
            note.append_page(v, Gtk.Label(s))
        # page[0]
        l01 = CLabel("<b>全体的な設定</b>")
        page[0].pack_start(l01, False, True, 0)
        self.cbox_pos = Gtk.CheckButton("終了時の位置を記憶復元する")
        self.cbox_pos.set_active(setting.position)
        page[0].pack_start(self.cbox_pos, False, True, 0)
        self.cbox_esc = Gtk.CheckButton("Esc で終了する")
        self.cbox_esc.set_active(setting.esc_exit)
        page[0].pack_start(self.cbox_esc, False, True, 0)
        self.cbox_memrep = Gtk.CheckButton("リピート状態を記憶する")
        self.cbox_memrep.set_active(setting.mem_rep)
        page[0].pack_start(self.cbox_memrep, False, True, 0)
        self.cbox_memaspect = Gtk.CheckButton("アスペクト比状態を記憶する")
        self.cbox_memaspect.set_active(setting.mem_aspect)
        page[0].pack_start(self.cbox_memaspect, False, True, 0)
        self.cbox_loop = Gtk.CheckButton("すべてリピートは最後の次は先頭へ")
        self.cbox_loop.set_active(setting.endressloop)
        page[0].pack_start(self.cbox_loop, False, True, 0)
        self.cbox_only = Gtk.CheckButton("常にウインドウサイズ")
        self.cbox_only.set_active(setting.size_win_only)
        page[0].pack_start(self.cbox_only, False, True, 0)
        #
        self.cbox_size = Gtk.CheckButton("起動時のサイズを以下に固定する")
        self.cbox_size.set_active(setting.init_size_on)
        self.combo_size = Gtk.ComboBoxText.new()
        for s in ("win","x1.0","x2.0","x3.0","x0.5","x1.5","x2.5","set1","set2","set3"):
            self.combo_size.append_text(s)
        self.combo_size.set_active(setting.init_size_val)
        hb1 = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        hb1.pack_start(self.cbox_size, False, True, 0)
        hb1.pack_end(self.combo_size, False, True, 0)
        page[0].pack_start(hb1, False, True, 0)
        #
        l02 = CLabel("マウスジェスチャ感度")
        self.sb_mouse = Gtk.SpinButton.new_with_range(0.0, 10000.0, 1.0)
        self.sb_mouse.set_value(setting.severity)
        hb2 = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        hb2.pack_start(l02, False, True, 0)
        hb2.pack_end(self.sb_mouse, False, True, 0)
        page[0].pack_start(hb2, False, True, 0)
        #
        # page[1]
        l11 = CLabel("<b>set size の指定</b>")
        page[1].pack_start(l11, False, True, 0)
        grid = Gtk.Grid.new() #Table(3, 4)
        self.sb = []
        for i in range(6):
            self.sb.append(Gtk.SpinButton.new_with_range(0.0, 10000.0, 1.0))
        self.sb[0].set_value(setting.set_size[0][0])
        self.sb[1].set_value(setting.set_size[0][1])
        self.sb[2].set_value(setting.set_size[1][0])
        self.sb[3].set_value(setting.set_size[1][1])
        self.sb[4].set_value(setting.set_size[2][0])
        self.sb[5].set_value(setting.set_size[2][1])
        table_label = ("width", "height", "set1", "set2", "set3")
        grid.attach(Gtk.Label(table_label[0]),1,0,1,1)
        grid.attach(Gtk.Label(table_label[1]),2,0,1,1)
        grid.attach(Gtk.Label(table_label[2]),0,1,1,1)
        grid.attach(Gtk.Label(table_label[3]),0,2,1,1)
        grid.attach(Gtk.Label(table_label[4]),0,3,1,1)
        grid.attach(self.sb[0],1,1,1,1)
        grid.attach(self.sb[1],2,1,1,1)
        grid.attach(self.sb[2],1,2,1,1)
        grid.attach(self.sb[3],2,2,1,1)
        grid.attach(self.sb[4],1,3,1,1)
        grid.attach(self.sb[5],2,3,1,1)
        page[1].pack_start(grid, False, True, 0)
        l12 = CLabel("<b>カスタムアスペクト比の指定</b>")
        self.entry_aspect = Gtk.Entry()
        self.entry_aspect.set_text("{0}:{1}".format(setting.aspect_rate[0], setting.aspect_rate[1]))
        page[1].pack_start(l12, False, True, 0)
        page[1].pack_start(self.entry_aspect, False, True, 0)
        #
        # page[2]
        l21 = CLabel("<b>リストアップ MIME Type の設定</b>")
        page[2].pack_start(l21, False, True, 0)
        frame = Gtk.Frame.new("改行区切りで指定")
        page[2].pack_start(frame, True, True, 0)
        mimebox = Gtk.TextView()
        self.mimebuf = mimebox.get_buffer()
        self.mimebuf.set_text(setting.mimes.replace("|", "\n"))
        sw = Gtk.ScrolledWindow()
        sw.add(mimebox)
        frame.add(sw)
        # pack main
        self.vbox.pack_start(note, True, True, 0)
        self.show_all()

if __name__ == "__main__":
    # Unconscientious debug
    import y901main
    y901main.main()
