#! /usr/bin/env python3

from gi.repository import Gtk, Gdk

class Y901ListBox(Gtk.Box):
    """
        GtkTreeView wrapping panels
    """
    def __init__(self, window):
        """
            @param window: Toplevel Window
            Synchronize Resize
        """
        self._win = window
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
        # private:
        self.__resize_w = 0
        self.__resize_x = 0
        self.__resize_y = 0
        self.__cx = 0
        self.__cy = 0
        # partition
        vs = Gtk.Separator.new(Gtk.Orientation.VERTICAL)
        vs.set_size_request(5, -1)
        self.pack_start(vs, False, True, 0)
        # on ScrolledWindow TreeView
        self.sw = Gtk.ScrolledWindow()
        self.sw.set_size_request(100, -1)
        self.sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        self.sw.set_shadow_type(Gtk.ShadowType.IN)
        # GtkTreeView
        self.view = Gtk.TreeView()
        self.sw.add(self.view)
        # GtkTreeColumn
        self.column1 = Gtk.TreeViewColumn()
        self.view.append_column(self.column1)
        self.connect("check-resize", self.__on_size_request)
        cell1 = Gtk.CellRendererText()
        self.column1.pack_start(cell1, 0)
        self.column1.add_attribute(cell1, "text", 0)
        # liststore create  
        self.liststore = Gtk.ListStore(str)
        self.view.set_model(self.liststore)
        self.view.set_headers_visible(False)
        self.selection = self.view.get_selection()
        self.selection.set_mode(Gtk.SelectionMode.SINGLE)
        # treeview pack
        self.pack_start(self.sw, True, True, 0)
        # DrawingArea
        self.dwa = Gtk.DrawingArea()
        self.dwa.set_events(Gdk.EventMask.EXPOSURE_MASK | 
                            Gdk.EventMask.POINTER_MOTION_MASK | 
                            Gdk.EventMask.POINTER_MOTION_HINT_MASK | 
                            Gdk.EventMask.BUTTON_PRESS_MASK | 
                            Gdk.EventMask.BUTTON_RELEASE_MASK | 
                            Gdk.EventMask.STRUCTURE_MASK )
                            #Gdk.EventMask.VISIBILITY_NOTIFY_MASK )
        self.dwa.set_size_request(10, -1)
        self.pack_end(self.dwa, False, True, 0)
        # Connect
        self.dwa.connect("button-press-event", self.__on_button_press_event)
        self.dwa.connect("motion-notify-event", self.__on_motion_notify_event)
        self.dwa.connect("button-release-event", self.__on_button_release_event)
        self.dwa.connect("draw", self.__on_draw_event)
        self.dwa.connect("realize", self.__on_realize)
        self.show_all()

    def is_text(self, text):
        return text in [ r[0] for r in self.liststore ]

    def select_text(self, text):
        model = self.view.get_model()
        it = model.get_iter_first()
        if it == None:
            self.selection.unselect_all()
        else:
            self._n = 0
            while 1:
                s = model.get_value(it, 0)
                if s == text:
                    self.selection.select_iter(it)
                    break
                it = model.iter_next(it)
                if it == None: break

    def selection_scroll(self):
        """
            Auto Scroll
        """
        # Get GtkAdjustment
        adjustment = self.view.get_vadjustment()
        # Is Select Check
        n = self.get_select_num()
        if n == -1:
            return
        page_size = adjustment.props.page_size
        column_height = self.column1.cell_get_size(None)[3]
        # calc
        sc = column_height * n
        # into Rect
        v = adjustment.get_value()
        h = page_size + v - (column_height * 2)
        if sc == 0:
            adjustment.set_value(0.0)
        elif sc < v:
            adjustment.set_value(sc - column_height)
        elif sc >= h:
            adjustment.set_value(sc - page_size + (column_height * 2))

    def get_select_num(self):
        model, it = self.selection.get_selected()
        if it:
            return model.get_path(it)[0]
        return -1

    def __on_size_request(self, widget, event=None):
        """
            Select Item View
        """
        self.selection_scroll()

    def __on_button_press_event(self, widget, event):
        # begin resize
        if event.button == 1:
            self.__resize_w = self.props.width_request
            self.__resize_x = event.x_root
            self.__cx, self.__cy = self._win.get_size()

    def __on_motion_notify_event(self, widget, event):
        # size change
        if self.__resize_w:
            x = event.x_root - self.__resize_x
            if self.__resize_w + x > 0:
                self.props.width_request = self.__resize_w + x
                self._win.resize(int(self.__cx + x), int(self.__cy))

    def __on_button_release_event(self, widget, event):
        # end resize
        if event.button == 1:
            self.__resize_w = 0
            self.dwa.queue_draw()

    def __on_draw_event(self, widget, cr):
        width = widget.get_allocated_width()
        height = widget.get_allocated_height()
        cr.set_source_rgb(1.0, 1.0, 1.0)
        cr.paint()
        cr.set_source_rgb(0.0, 0.0, 0.0)
        cr.set_line_width(0.2)
        for i in range(10):
            if i%3 == 0:
                cr.move_to(i, 0)
                cr.line_to(i, height)
                cr.stroke()

    def __on_realize(self, widget, event=None):
        cur_grip = Gdk.Cursor(Gdk.CursorType.RIGHT_TEE)
        self.dwa.props.window.set_cursor(cur_grip)

