#!/usr/bin/env python3

import sys
import gi
try:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Gst", "1.0")
    gi.require_version("GstAudio", "1.0")
    gi.require_version("GdkX11", "3.0")
    gi.require_version("GstVideo", "1.0")
except Exception as e:
    print("Error: {0}".format(e))
    sys.exit()

from gi.repository import Gtk, Gst, GstAudio, GLib, Gdk, GdkPixbuf, GObject, Gio, GdkX11, GstVideo

import os
import time
import optparse
import math
#
import inifile8
import y901dialog
from common import *
import numsort
import y901menubtn
import y901statusbar
import y901headerbar
import y901listbox
import y901toolbox
import y901fullscreentool

VERSION = "1.1.3"

class Y901window(Gtk.ApplicationWindow):
    """
        Main Window Class
    """
    def __init__(self, app):
        """
            Create Y901x Window
        """
        sys.path.insert(0, GLib.path_get_dirname(__file__))
        Gtk.ApplicationWindow.__init__(self, application=app)
        # attribute
        self.playinfo = CPlayInfo()
        self.path = CPath()
        self.seekinfo = CSeekInfo()
        self.timer_id = -1
        self.rlist = CRList()
        self.fullinfo = CFullInfo()
        self.settingwin = CSettingWin()
        self.setting = CSetting()
        self.mime_list = []
        self.gesture = CGesture()
        self.mousepoint = CPoint()
        self.invisiblecount = 0
        # Read config file
        inipath = "{0}/sasakima/y901x.conf".format(GLib.get_user_config_dir())
        if GLib.file_test(inipath, GLib.FileTest.EXISTS):
            ini = inifile8.Inifile(inipath)
            try:
                # [window]
                x = ini.read_int("window", "left", 0)
                y = ini.read_int("window", "top", 0)
                cx = ini.read_int("window", "width", 320)
                cy = ini.read_int("window", "height", 320)
                self.settingwin.list_width  = ini.read_int("window", "list_width", 100)
                self.settingwin.list_show   = ini.read_bool("window", "list_show", True)
                self.settingwin.volume      = ini.read_float("window", "volume", 100.0)
                self.settingwin.video_size  = ini.read_int("window", "video_size", 0)
                self.settingwin.repeat_mem  = ini.read_int("window", "repeat", 0)
                self.settingwin.aspect_mem  = ini.read_int("window", "aspect", 0)
                self.settingwin.n_audio     = ini.read_int("window", "n_audio", 0)
                # [setting]
                self.setting.position       = ini.read_bool("setting", "position", False)
                self.setting.esc_exit       = ini.read_bool("setting", "esc_exit", False)
                self.setting.mem_rep        = ini.read_bool("setting", "mem_rep", False)
                self.setting.mem_aspect     = ini.read_bool("setting", "mem_aspect", False)
                self.setting.endressloop    = ini.read_bool("setting", "loop", False)
                self.setting.size_win_only  = ini.read_bool("setting", "size_win_only", False)
                self.setting.init_size_on   = ini.read_bool("setting", "init_size_on", False)
                self.setting.init_size_val  = ini.read_int("setting", "init_size_val", 0)
                self.setting.severity       = ini.read_int("setting", "severity", 25)
                self.setting.mimes          = ini.read_str("setting", "mimes", Y901_MIMES)
                self.setting.set_size[0][0] = ini.read_int("setting", "set_size1", 320)
                self.setting.set_size[0][1] = ini.read_int("setting", "set_size2", 240)
                self.setting.set_size[1][0] = ini.read_int("setting", "set_size3", 640)
                self.setting.set_size[1][1] = ini.read_int("setting", "set_size4", 480)
                self.setting.set_size[2][0] = ini.read_int("setting", "set_size5", 1280)
                self.setting.set_size[2][1] = ini.read_int("setting", "set_size6", 720)
                self.setting.aspect_rate[0] = ini.read_int("setting", "aspect_rate0", 1)
                self.setting.aspect_rate[1] = ini.read_int("setting", "aspect_rate1", 1)
            except Exception as s:
                self.messagebox("{0}".format(s))
                sys.exit()
            # resize and move
            if self.setting.init_size_on:
                self.settingwin.video_size = self.setting.init_size_val
            self.resize(cx, cy)
            if self.setting.position:
                self.move(x, y)
        # List up the MIME Types
        self.mime_list = self.setting.mimes.split("|")
        # GtkAccelGroup (accel_key, accel_mods, accel_flags, callback)
        accelgroup = Gtk.AccelGroup.new()
        accelgroup.connect(Gdk.KEY_KP_0, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_1, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_2, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_3, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_4, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_5, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_6, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_7, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_8, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        accelgroup.connect(Gdk.KEY_KP_9, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        #
        accelgroup.connect(Gdk.KEY_plus, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_minus, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_equal, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_KP_Add, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_KP_Subtract, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_KP_Enter, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        #
        accelgroup.connect(Gdk.KEY_F2, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_F5, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_F7, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_F8, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_F9, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_F11, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_F12, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_Return, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_space, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_BackSpace, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_Escape, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_Delete, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_Delete, Gdk.ModifierType.SHIFT_MASK, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        accelgroup.connect(Gdk.KEY_Delete, Gdk.ModifierType.CONTROL_MASK, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.add_accel_group(accelgroup)
        # HeaderBar
        self.headerbar = y901headerbar.Y901HeaderBar()
        self.headerbar.create_sizemenu(accelgroup, Y901_SIZE_STRS, Y901_SIZE_KEYS, self.on_sizemenu)
        self.headerbar.create_aspectmenu(Y901_ASPECT_STRS, self.on_aspectmenu)
        self.headerbar.fullbutton.connect("clicked", self.on_full)
        self.headerbar.create_preferencebutton(accelgroup, Y901_MENU_STRS, Y901_MENU_KEYS, self.on_preference)
        self.headerbar.preferencebutton.add_accelerator("clicked", accelgroup, Gdk.KEY_F10, 0, Gtk.AccelFlags.VISIBLE)
        # StatusBar
        self.speedbutton = y901menubtn.Y901MiniMenuButton(Y901_SPEED_LABELS, 5, self.set_playback_speed)
        self.audiobutton = y901menubtn.Y901MiniMenuButton(["Track #1"], 0, self.set_audio_track)
        self.statusbar = y901statusbar.Y901StatusBar(self, self.speedbutton, self.audiobutton)
        # ListBox
        self.listbox = y901listbox.Y901ListBox(self)
        self.listbox.set_size_request(self.settingwin.list_width, -1)
        self.listbox.view.connect("button-release-event", self.on_listbox_button_release)
        self.listbox.props.visible = self.settingwin.list_show
        # ToolBox
        self.toolbox = y901toolbox.Y901ToolBox()
        self.toolbox.play.connect("clicked", self.on_play)
        self.toolbox.stop.connect("clicked", self.on_stop)
        self.toolbox.playlist.connect("clicked", self.on_list_button_clicked)
        self.toolbox.seekbar.connect("value-changed", self.on_seek_value_changed)
        self.toolbox.seekbar.connect("button-press-event", self.on_seek_button_press_event)
        self.toolbox.seekbar.connect("button-release-event", self.on_seek_button_release_event)
        self.toolbox.seekbar.connect("scroll-event", self.on_seek_scroll_event)
        self.toolbox.volumebar.connect("button-release-event", self.on_volume_button_release_event)
        self.toolbox.one.connect("clicked", self.on_rep_btn, 1)
        self.toolbox.all.connect("clicked", self.on_rep_btn, 2)
        self.toolbox.rdm.connect("clicked", self.on_rep_btn, 3)
        self.toolbox.volumeadj.set_value(self.settingwin.volume)
        self.toolbox.volumebar.props.sensitive = False
        # PlayBin2
        self.player = Gst.ElementFactory.make("playbin", None)
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.gst_on_message)
        bus.connect("sync-message::element", self.gst_on_sync_message)
        # Container
        vbox_submain = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        vbox_submain.show()
        self.hbox_ctrl = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.hbox_ctrl.show()
        self.vbox_main = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.vbox_main.show()
        # Video Window, Background Color to Black
        self.video_window = Gtk.DrawingArea()
        self.video_window.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0,0,0,1))
        self.video_window.props.can_focus = True
        # GtkAspectFrame, draw a Background Color to Black
        self.aframe = Gtk.AspectFrame.new("null", 0.5, 0.5, 4.0/3.0, False)
        #self.aframe.set_label_align(0.0, 0.0)
        self.aframe.props.label = None
        self.aframe.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0,0,0,1))
        self.aframe.set_shadow_type(Gtk.ShadowType.NONE)
        self.aframe.set_events(
                Gdk.EventMask.EXPOSURE_MASK |
                Gdk.EventMask.STRUCTURE_MASK |
                Gdk.EventMask.VISIBILITY_NOTIFY_MASK)
        self.aframe.connect("draw", self.on_aframe_draw)
        self.aframe.add(self.video_window)
        # packing
        self.hbox_ctrl.pack_start(self.headerbar, False, True, 0)
        self.hbox_ctrl.pack_start(self.aframe, True, True, 0)
        self.hbox_ctrl.pack_end(self.toolbox, False, True, 0)
        self.hbox_ctrl.show_all()
        vbox_submain.pack_start(self.hbox_ctrl, True, True, 0)
        vbox_submain.pack_end(self.listbox, False, True, 0)
        self.vbox_main.pack_start(vbox_submain, True, True, 0)
        self.vbox_main.pack_start(self.statusbar, False, False, 0)
        self.add(self.vbox_main)
        # TitleBar
        iconpath = GLib.build_filenamev([GLib.path_get_dirname(__file__), "y901x.png"])
        self.y901icon = GdkPixbuf.Pixbuf.new_from_file(iconpath)
        self.set_icon(self.y901icon)
        self.set_title("Y901x")
        # Drop source set
        dnd_list = Gtk.TargetEntry.new("text/uri-list", 0, 0)
        self.drag_dest_set(
                Gtk.DestDefaults.MOTION |
                Gtk.DestDefaults.HIGHLIGHT |
                Gtk.DestDefaults.DROP,
                [dnd_list],
                Gdk.DragAction.MOVE )
        self.drag_dest_add_uri_targets()
        # GDK event set
        self.set_events(
                Gdk.EventMask.KEY_PRESS_MASK |
                Gdk.EventMask.KEY_RELEASE_MASK |
                Gdk.EventMask.STRUCTURE_MASK )
        self.video_window.set_events(
                Gdk.EventMask.LEAVE_NOTIFY_MASK |
                Gdk.EventMask.BUTTON_PRESS_MASK |
                Gdk.EventMask.BUTTON_RELEASE_MASK |
                Gdk.EventMask.POINTER_MOTION_MASK |
                Gdk.EventMask.POINTER_MOTION_HINT_MASK )
        # Create invisible cursor
        self.noncursor = Gdk.Cursor.new(Gdk.CursorType.BLANK_CURSOR)
        # Signals
        self.video_window.connect("button-press-event", self.on_button_press)
        self.video_window.connect("button-release-event", self.on_button_release)
        self.video_window.connect("motion-notify-event", self.on_motion_notify)
        self.connect("drag-data-received", self.on_drag_data_received)
        self.connect("key-press-event", self.on_key_press)
        self.connect("key-release-event", self.on_key_release)
        self.show()
        # Under the Show After
        self.xid = self.video_window.props.window.get_xid()
        # Size menu position changes. Throw on_size_change signal.
        self.headerbar.sizemenu.get_children()[self.settingwin.video_size].activate()
        # Repert memory Attach
        if self.setting.mem_rep:
            self.on_rep_btn(None, self.settingwin.repeat_mem)
        # Aspect rate Attach
        if self.setting.mem_aspect:
            action3 = self.actiongroup.get_action(ac3_str[self.settingwin.aspect_mem])
            action3.set_current_value(self.settingwin.aspect_mem)
        # move focus
        self.set_focus(self.video_window)
        # change gstreamer volume
        self.toolbox.volumebar.connect("value-changed", self.on_volume_value_changed)
        # argv processing is main function

########################################################################
#
#                       Functions
#
########################################################################

    def do_delete_event(self, event):
        app = self.get_application()
        array = app.get_windows()
        if len(array) == 1:
            self.player.set_state(Gst.State.NULL)
            # Search "y901x.conf"
            direpath = GLib.get_user_config_dir() + "/sasakima"
            directory = Gio.file_new_for_path(direpath)
            if not directory.query_exists(None):
                directory.make_directory(None)
            confpath = direpath + "/y901x.conf"
            ini = inifile8.Inifile(confpath)
            try:
                # [window]
                x, y = self.get_position()
                ini.write_int("window", "left", x)
                ini.write_int("window", "top", y)
                if self.fullinfo.is_fullscreen:
                    ini.write_int("window", "width", self.fullinfo.width)
                    ini.write_int("window", "height", self.fullinfo.height)
                else:
                    cx, cy = self.get_size()
                    ini.write_int("window", "width", cx)
                    ini.write_int("window", "height", cy)
                ini.write_int("window", "list_width", self.listbox.props.width_request)
                ini.write_bool("window", "list_show", self.listbox.props.visible)
                ini.write_float("window", "volume", self.toolbox.volumeadj.get_value())
                ini.write_int("window", "video_size", self.settingwin.video_size)
                ini.write_int("window", "repeat", self.settingwin.repeat)
                ini.write_int("window", "aspect", self.settingwin.aspect)
                ini.write_int("window", "n_audio", self.settingwin.n_audio)
                # [setting]
                ini.write_bool("setting", "position", self.setting.position)
                ini.write_bool("setting", "esc_exit", self.setting.esc_exit)
                ini.write_bool("setting", "mem_rep", self.setting.mem_rep)
                ini.write_bool("setting", "mem_aspect", self.setting.mem_aspect)
                ini.write_bool("setting", "loop", self.setting.endressloop)
                ini.write_bool("setting", "size_win_only", self.setting.size_win_only)
                ini.write_bool("setting", "init_size_on", self.setting.init_size_on)
                ini.write_int("setting", "init_size_val", self.setting.init_size_val)
                ini.write_int("setting", "severity", self.setting.severity)
                ini.write_str("setting", "mimes", self.setting.mimes)
                ini.write_int("setting", "set_size1", self.setting.set_size[0][0])
                ini.write_int("setting", "set_size2", self.setting.set_size[0][1])
                ini.write_int("setting", "set_size3", self.setting.set_size[1][0])
                ini.write_int("setting", "set_size4", self.setting.set_size[1][1])
                ini.write_int("setting", "set_size5", self.setting.set_size[2][0])
                ini.write_int("setting", "set_size6", self.setting.set_size[2][1])
                ini.write_int("setting", "aspect_rate0", self.setting.aspect_rate[0])
                ini.write_int("setting", "aspect_rate1", self.setting.aspect_rate[1])
                ini.save()
            except Exception as s:
                self.messagebox("{0}".format(s))

    def messagebox(self, text, icon=Gtk.MessageType.WARNING, button=Gtk.ButtonsType.OK):
        """
            Simple MessageBox
        """
        dlg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, icon, button, text)
        dlg.set_title("from Y901x")
        r = dlg.run()
        dlg.destroy()
        return r

    def messagebox_trash(self, text):
        """
            Trash Only MessageBox
        """
        dlg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.QUESTION,  Gtk.ButtonsType.NONE, text)
        dlg.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
        dlg.add_button("ゴミ箱へ移動する", Gtk.ResponseType.OK)
        dlg.set_default_response(Gtk.ResponseType.OK)
        dlg.set_title("from Y901x")  
        r = dlg.run()  
        dlg.destroy()
        return r

    def set_uri(self, uri):
        """
            Once NULL.
            Non supported file => gst_on_message
        """
        if uri == None:
            return
        self.clear()
        self.player.set_state(Gst.State.NULL)
        self.toolbox.set_enable_ctrl(False)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.playinfo.uri = uri # v3.6
        self.player.props.uri = uri
        self.player.set_state(Gst.State.PLAYING)
        self.set_focus(self.video_window)

    def close(self):
        """
            Application Quit and Setting File Write
        """
        #Gtk.main_quit()
        self.emit("delete-event", Gdk.Event(Gdk.EventType.DELETE))
        self.props.application.remove_window(self)

    def set_next_frame(self, rate):
        """
            1 frame up down
        """
        if self.settingwin.direction != rate:
            self.settingwin.direction = rate
            self.set_playback_direction(rate)
        event = Gst.Event.new_step(Gst.Format.BUFFERS, 1, 1.0, True, False)
        self.player.send_event(event)

    def set_playback_direction(self, rate):
        """
            Change direction (rate 1.0 or -1.0)
        """
        pos = self.player.query_position(Gst.Format.TIME)[1]
        if rate >= 0.0:
            self.player.seek(
                    rate,
                    Gst.Format.TIME,
                    Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                    Gst.SeekType.SET, pos,
                    Gst.SeekType.SET, -1)
        else:
            self.player.seek(
                    rate,
                    Gst.Format.TIME,
                    Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                    Gst.SeekType.SET, 0,
                    Gst.SeekType.SET, pos)

    def set_playback_speed(self, n):
        """
            http://docs.gstreamer.com/display/GstSDK/Basic+tutorial+13%3A+Playback+speed
        """
        pos = self.player.query_position(Gst.Format.TIME)[1]
        self.settingwin.speed = n
        if Y901_SPEED_NUMS[n] > 0:
            event = Gst.Event.new_seek(
                    Y901_SPEED_NUMS[n],
                    Gst.Format.TIME,
                    Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                    Gst.SeekType.SET, pos,
                    Gst.SeekType.NONE, 0)
        else:
            event = Gst.Event.new_seek(
                    Y901_SPEED_NUMS[n],
                    Gst.Format.TIME,
                    Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                    Gst.SeekType.SET, 0,
                    Gst.SeekType.SET, pos)
        self.player.send_event(event)

    def set_audio_track(self, n):
        """
            Multi Track Audio Select
        """
        self.player.props.current_audio = n
        self.settingwin.n_audio = n

    def set_play_position(self, location):
        """
            Media Seeking
            SeekFlags.KEY_UNIT -> LightWaight
        """
        self.player.seek(
                Y901_SPEED_NUMS[self.settingwin.speed],
                Gst.Format.TIME,
                Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                Gst.SeekType.SET, location,
                Gst.SeekType.NONE, 0)

    def clear(self):
        """
            State Non Playng
        """
        try:
            self.playinfo.position = 0
            self.playinfo.is_playing = False;
            if self.timer_id != -1:
                #self.timer_id = -1
                GLib.source_remove(self.timer_id)
                self.timer_id = -1
            if self.playinfo.duration != Gst.CLOCK_TIME_NONE:
                self.player.set_state(Gst.State.READY)
                self.toolbox.seekadj.set_value(0)
                allsec = self.playinfo.duration // 1000000000
                secv = "00:00/{0:02d}:{1:02d}".format((allsec // 60), (allsec % 60))
                self.statusbar.timelabel.set_text(secv)
                self.toolbox.change_img(1)
            self.video_window.set_double_buffered(True)
            self.video_window.props.window.set_cursor(None)
        except Exception as e:
            self.messagebox(e)

    def change_video_size(self):
        """
            Video Window Size
        """
        if self.settingwin.video_size < 7:
            if self.playinfo.uri == None: # First start and not set_size
                return
        self.set_focus(self.video_window)
        y = self.playinfo.vsize.height
        # Vertical reference
        if self.settingwin.aspect == 0:
            asp = self.playinfo.vsize.width
        elif self.settingwin.aspect == 1:
            asp = self.playinfo.vsize.height * 4 / 3
        elif self.settingwin.aspect == 2:
            asp = self.playinfo.vsize.height * 16 / 9
        elif self.settingwin.aspect == 3:
            asp = self.playinfo.vsize.height * self.setting.aspect_rate[0] / self.setting.aspect_rate[1]
        # aspect rate change
        self.aframe.set(0.5, 0.5, asp / y, False)
        # window size @ return
        if self.settingwin.video_size == 0:
            return
        x = int(asp)
        # calc
        menu_v = self.headerbar.get_allocated_height()
        ctrl_v = self.toolbox.get_allocated_height()
        stat_v = self.statusbar.get_allocated_height()
        if self.settingwin.video_size == 2: # x2.0
            x *= 2
            y *= 2
        elif self.settingwin.video_size == 3: # x3.0
            x *= 3
            y *= 3
        elif self.settingwin.video_size == 4: # x0.5
            x //= 2
            y //= 2
        elif self.settingwin.video_size == 5: # x1.5
            x = x * 3 // 2
            y = y * 3 // 2
        elif self.settingwin.video_size == 6: # x2.5
            x = x * 5 // 2
            y = y * 5 // 2
        elif self.settingwin.video_size == 7: # set1
            x = self.setting.set_size[0][0]
            y = self.setting.set_size[0][1]
        elif self.settingwin.video_size == 8: # set2
            x = self.setting.set_size[1][0]
            y = self.setting.set_size[1][1]
        elif self.settingwin.video_size == 9: # set3
            x = self.setting.set_size[2][0]
            y = self.setting.set_size[2][1]
        if x ==0 or y == 0:
            return
        # Add menu and controls size
        if self.listbox.props.visible:
            x += self.listbox.props.width_request
        y += menu_v + ctrl_v + stat_v
        # Compare Current Size
        cx, cy = self.get_size()
        if cx != x or cy != y:
            self.resize(x, y)
        # Back to WindowSize
        if self.setting.size_win_only:
            self.settingwin.video_size = 0
            self.headerbar.sizemenu.get_children()[0].activate()

    def listup_change(self):
        """
            FileList Re-reading
        """
        if self.playinfo.uri == None:
            return
        t = GLib.filename_from_uri(self.playinfo.uri)[0]
        dirc = GLib.path_get_dirname(t)
        self.path.name = GLib.path_get_basename(t)
        if self.path.dirc != dirc or self.listbox.is_text(self.path.name) == False:
            self.path.dirc = dirc
            self.listbox.liststore.clear()
            self.rlist.clear()
            files = numsort.get_file_numsort_list(self.path.dirc)
            for f in files:
                name = GLib.build_filenamev([self.path.dirc, f])
                gvfs = Gio.File.new_for_path(name)
                info = gvfs.query_info(
                        Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                        Gio.FileQueryInfoFlags.NONE,
                        None)
                mime = info.get_content_type()
                # Add Match MIME Type and Drop File
                if mime in self.mime_list or f == self.path.name:
                    it = self.listbox.liststore.append([f])
                    self.rlist.append(f)
                    # select change
                    if f == self.path.name:
                        self.listbox.select_text(self.path.name)
        else:
            # if Same Directory File Drop
            self.listbox.select_text(self.path.name)
        self.set_title("{0} ({1}/{2}) - Y901x".format(
                self.path.name,
                self.listbox.get_select_num() + 1,
                len(self.listbox.liststore))
            )
        # Create Random List
        self.rlist.shuffle()
        self.listbox.selection_scroll()

    def rename(self):
        """
            Rename the Playng File
        """
        if self.path.name == "":
            return
        label = Gtk.Label(self.path.name)
        entry = Gtk.Entry()
        entry.set_text(self.path.name)
        d = Gtk.Dialog( "リネーム",
                        self,
                        Gtk.DialogFlags.MODAL,
                        (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
                        Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT) )
        try:
            d.vbox.pack_start(label, False, True, 0)
            d.vbox.pack_start(entry, False, True, 0)
            d.show_all()
            def dlg_ok(self):
                d.response(Gtk.ResponseType.ACCEPT)
            def on_focus_in(self, widget):
                try:
                    s = entry.get_text()
                    i = s.rindex(".")
                    entry.select_region(0, i)
                except ValueError:
                    pass # is SelectAll
            def on_focus_out(self, widget):
                entry.select_region(0, 0)
            entry.connect("activate", dlg_ok)
            entry.connect("focus-in-event", on_focus_in)
            entry.connect("focus-out-event", on_focus_out)
            # Playing?
            is_play = False
            if self.player.get_state(1)[1] == Gst.State.PLAYING:
                is_play = True
                self.player.set_state(Gst.State.PAUSED)
            # Until the Success or Cancel.
            while 1:
                if d.run() == Gtk.ResponseType.ACCEPT:
                    text = entry.get_text()
                    if text == self.path.name:
                        self.messagebox("変更されていません")
                    else:
                        if  text in os.listdir(self.path.dirc):
                            self.messagebox("同一ファイル名が見つかりました")
                        else:
                            newname = GLib.build_filenamev([self.path.dirc, text])
                            self.clear()
                            while Gtk.events_pending():
                                Gtk.main_iteration()
                            os.rename(GLib.build_filenamev([self.path.dirc, self.path.name]), newname)
                            while Gtk.events_pending():
                                Gtk.main_iteration()
                            self.set_uri(GLib.filename_to_uri(newname, None))
                            return
                else:
                    break
        finally:
            d.destroy()
        if is_play:
            self.player.set_state(Gst.State.PLAYING)

    def trash(self):
        """
            Trash the Playng File
        """
        if self.path.name == "":
            return
        # Playing
        is_play = False
        if self.player.get_state(1)[1] == Gst.State.PLAYING:
            is_play = True
            self.player.set_state(Gst.State.PAUSED)
        r = self.messagebox_trash("再生中ファイルをゴミ箱に捨てます\nよろしいですか？")
        if r == Gtk.ResponseType.OK:
            fname = self.get_next_list()
            newname = GLib.build_filenamev([self.path.dirc, fname])
            self.clear()
            while Gtk.events_pending():
                Gtk.main_iteration()
            # Trash
            obj = Gio.File.new_for_path(GLib.build_filenamev([self.path.dirc, self.path.name]))
            try:
                res = obj.trash(None)
            except Exception as s:
                self.messagebox("{0}".format(s))
                res = False
            if res:
                while Gtk.events_pending():
                    Gtk.main_iteration()
                self.path.dirc = ""
                if fname != "":
                    self.set_uri(GLib.filename_to_uri(newname, None))
                else:
                    self.path.name = ""
                    self.listbox.liststore.clear()
                return
        if is_play:
            self.player.set_state(Gst.State.PLAYING)

    def trash_next(self):
        """
            Trash the Playng Under File
        """
        if self.path.name == "":
            return
        model, it = self.listbox.selection.get_selected()
        if it:
            it2 = model.iter_next(it)
            if it2:
                fname = model.get_value(it2, 0)
                delname = GLib.build_filenamev([self.path.dirc, fname])
                obj = Gio.File.new_for_path(delname)
                try:
                    res = obj.trash(None)
                    self.path.dirc = ""
                    self.listup_change()
                except Exception as s:
                    self.messagebox("{0}".format(s))

    def trash_prev(self):
        """
            Trash the Playng Upper File
        """
        if self.path.name == "":
            return
        model, it = self.listbox.selection.get_selected()
        if it:
            row = model.get_path(it)[0] - 1
            if row > -1:
                it2 = model.get_iter((row, ))
                fname = model.get_value(it2, 0)
                delname = GLib.build_filenamev([self.path.dirc, fname])
                obj = Gio.File.new_for_path(delname)
                try:
                    res = obj.trash(None)
                    self.path.dirc = ""
                    self.listup_change()
                except Exception as s:
                    self.messagebox("{0}".format(s))

    def get_next_list(self):
        """
            Listup Next Select Search
        """
        s = ""
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return s
        # next
        it2 = model.iter_next(it)
        if it2 == None:
            # no next
            row = model.get_path(it)[0] - 1
            if row < 0:
                return s
            self.listbox.selection.select_path(row)
            it3 = self.listbox.selection.get_selected()[1]
            s = model.get_value(it3, 0)
        else:
            self.listbox.selection.select_iter(it2)
            s = model.get_value(it2, 0)
        return s

    def change_fullscreen(self):
        """
            Switching FullScreen
        """
        if self.path.name == "":
            return
        if self.fullinfo.is_fullscreen:
            self.fullinfo.is_fullscreen = False
            self.props.window.unfullscreen()
            self.fullinfo.toolbox.destroy()
            self.fullinfo.toolbox = None
            self.headerbar.show()
            if self.fullinfo.is_show_list:
                self.listbox.show()
        else:
            self.fullinfo.is_fullscreen = True
            cx, cy = self.get_size()
            self.fullinfo.width = cx
            self.fullinfo.height = cy
            self.fullinfo.is_show_list = self.listbox.props.visible
            if self.fullinfo.is_show_list:
                self.listbox.hide()
            self.headerbar.hide()
            self.fullinfo.toolbox = y901fullscreentool.Y901FullScreenTool(
                    self.hbox_ctrl,
                    self.toolbox,
                    self.vbox_main,
                    self.statusbar
                )
            self.props.window.fullscreen()


########################################################################
#
#                   Menu & Button Signal Handlers
#
########################################################################


    def do_open(self):
        dlg = Gtk.FileChooserDialog("ファイルを開く", self,
                                    Gtk.FileChooserAction.OPEN,
                                    (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, 
                                    Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        # Set MIME Types Filter
        filt = Gtk.FileFilter()
        filt.set_name("Support file")
        for mimestring in self.mime_list:
            filt.add_mime_type(mimestring)
        dlg.add_filter(filt)
        filt = Gtk.FileFilter()
        filt.set_name("All files")
        filt.add_pattern("*")
        dlg.add_filter(filt)
        # Set Open Directry
        if self.path.dirc == "":
            # search VIDEOS Directry
            sd = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_VIDEOS)
            if sd:
                dlg.set_current_folder(sd)
            else:
                dlg.set_current_folder(GLib.get_home_dir())
        else:
            dlg.set_current_folder(self.path.dirc)
        r = dlg.run()
        if r == Gtk.ResponseType.OK:
            self.set_uri(GLib.filename_to_uri(dlg.get_filename(), None))
        dlg.destroy()

    def on_play(self, widget, data=None):
        if self.settingwin.direction == -1.0:
            self.settingwin.direction = 1.0
            self.set_playback_direction(1.0)
        if self.player.get_state(1)[1] == Gst.State.PLAYING:
            self.player.set_state(Gst.State.PAUSED)
        else:
            self.player.set_state(Gst.State.PLAYING)

    def on_stop(self, widget, data=None):
        self.clear()

    def do_prev(self):
        if self.settingwin.repeat == 3:
            n = len(self.rlist.l)
            if n == 0:
                return
            elif n == 1:
                self.set_play_position(0)
                self.player.set_state(Gst.State.PLAYING)
            else:
                self.set_uri(GLib.filename_to_uri("{0}/{1}".format(self.path.dirc, self.rlist.get()), None))
        else:
            model, it = self.listbox.selection.get_selected()
            if it == None:
                return
            row = model.get_path(it)[0] - 1
            if row < 0:
                return
            # Change Listbox Select
            self.listbox.selection.select_path(row)
            it = self.listbox.selection.get_selected()[1]
            s = model.get_value(it, 0)
            self.set_uri(GLib.filename_to_uri("{0}/{1}".format(self.path.dirc, s), None))

    def do_next(self):
        if self.settingwin.repeat == 3:
            n = len(self.rlist.l)
            if n == 0:
                return
            elif n == 1:
                self.set_play_position(0)
                self.player.set_state(Gst.State.PLAYING)
            else:
                self.set_uri(GLib.filename_to_uri("{0}/{1}".format(self.path.dirc, self.rlist.get()), None))
        else:
            # is endressloop ?
            model, it = self.listbox.selection.get_selected()
            if it == None:
                return
            it = model.iter_next(it)
            if it == None:
                if self.setting.endressloop:
                    it = model.get_iter_first()
                else:
                    return
            # Change Listbox Select
            self.listbox.selection.select_iter(it)
            s = model.get_value(it, 0)
            self.set_uri(GLib.filename_to_uri("{0}/{1}".format(self.path.dirc, s), None))

    def on_rep_btn(self, widget, num):
        if widget.get_active():
            if self.settingwin.repeat != num:
                self.toolbox.one.set_active(num == 1)
                self.toolbox.all.set_active(num == 2)
                self.toolbox.rdm.set_active(num == 3)
                self.settingwin.repeat = num
        else:
            if not self.toolbox.one.get_active() and not self.toolbox.all.get_active() and not self.toolbox.rdm.get_active():
                self.settingwin.repeat = 0

    def on_full(self, widget, data=None):
        self.change_fullscreen()

    def on_aspectmenu(self, widget, num):
        if self.settingwin.aspect != num:
            self.settingwin.aspect = num
            self.headerbar.aspectbutton.set_label(Y901_ASPECT_STRS[num])
            self.change_video_size()

    def on_sizemenu(self, widget, num):
        # Release FullScreen
        if self.fullinfo.is_fullscreen:
            self.change_fullscreen()
        self.settingwin.video_size =  num
        self.headerbar.sizebutton.set_label(Y901_SIZE_STRS[num])
        self.change_video_size()

    def on_preference(self, widget, num):
        if num == 0:
            self.do_open()
        elif num == 1:
            self.do_setting()
        elif num == 2:
            self.close()

    def do_setting(self):
        dlg = y901dialog.SettingDialog(self, self.setting)
        try:
            if dlg.run() == Gtk.ResponseType.ACCEPT:
                self.setting.position      = dlg.cbox_pos.get_active()
                self.setting.esc_exit      = dlg.cbox_esc.get_active()
                self.setting.mem_rep       = dlg.cbox_memrep.get_active()
                self.setting.mem_aspect    = dlg.cbox_memaspect.get_active()
                self.setting.endressloop   = dlg.cbox_loop.get_active()
                self.setting.size_win_only = dlg.cbox_only.get_active()
                self.setting.init_size_on  = dlg.cbox_size.get_active()
                self.setting.init_size_val = dlg.combo_size.get_active()
                self.setting.severity = int(dlg.sb_mouse.get_value())
                startiter = dlg.mimebuf.get_start_iter()
                enditer   = dlg.mimebuf.get_end_iter()
                mimes = dlg.mimebuf.get_text(startiter, enditer, True).replace("\n", "|")
                if mimes != self.setting.mimes:
                    self.setting.mimes = mimes
                    self.mime_list = mimes.split("|")
                    self.path.dirc = ""
                    self.listup_change()
                self.setting.set_size[0][0] = int(dlg.sb[0].get_value())
                self.setting.set_size[0][1] = int(dlg.sb[1].get_value())
                self.setting.set_size[1][0] = int(dlg.sb[2].get_value())
                self.setting.set_size[1][1] = int(dlg.sb[3].get_value())
                self.setting.set_size[2][0] = int(dlg.sb[4].get_value())
                self.setting.set_size[2][1] = int(dlg.sb[5].get_value())
                aspect = dlg.entry_aspect.get_text().split(":")
                self.setting.aspect_rate[0] = int(aspect[0])
                self.setting.aspect_rate[1] = int(aspect[1])
                if self.setting.init_size_val:
                    self.settingwin.video_size = 0
                    self.headerbar.sizemenu.get_children()[0].activate()
        except Exception as s:
            self.messagebox("{0}".format(s))
        finally:
            dlg.destroy()
        self.set_focus(self.video_window)

    def on_list_button_clicked(self, widget, data=None):
        if self.fullinfo.is_fullscreen:
            return
        width = self.listbox.props.width_request
        cx, cy = self.get_size()
        if self.listbox.props.visible:
            self.listbox.hide()
            self.resize(cx - width, cy)
        else:
            self.listbox.show()
            self.resize(cx + width, cy)


########################################################################
#
#                   MainWindow Signal Handlers
#
########################################################################

    
    def on_button_press(self, widget, event, data=None):
        """
            Start Mouse Gesture and W Click
        """
        if event.button == 3:
            self.gesture.start = True
            self.gesture.x = event.x_root
            self.gesture.y = event.y_root
            self.gesture.result = ""
            self.gesture.prevchar = ""
            self.gesture.title = ""
        elif event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            self.change_fullscreen()
    
    def on_motion_notify(self, widget, event, data=None):
        """
            Mouse Gesture
        """
        if self.gesture.start:
            x = abs(event.x_root - self.gesture.x)
            y = abs(event.y_root - self.gesture.y)
            s = ""
            if (x > self.setting.severity) or (y > self.setting.severity):
                if x > y:
                    if (self.gesture.x - event.x_root) > 0:
                        s = "←"
                    else:
                        s = "→"
                else:
                    if (self.gesture.y - event.y_root) > 0:
                        s = "↑"
                    else:
                        s = "↓"
                if self.gesture.prevchar != s:
                    self.gesture.prevchar = s
                    self.gesture.result += s
                self.gesture.x = event.x_root
                self.gesture.y = event.y_root
                if self.gesture.title == "":
                    self.gesture.title = self.get_title()
                self.set_title(self.gesture.result)
    
    def on_button_release(self, widget, event, data=None):
        """
            Mouse Gesture
            if event.button == 3: is no
        """
        if self.gesture.start:
            self.gesture.start = False
            # Separation (Prevent PopupMenu)
            if self.gesture.result == "" and self.get_title() != "":
                pass #self.popup_menu.popup(None, None, None, None, 0, event.time)
            else:
                if self.gesture.title != "":
                    self.set_title(self.gesture.title)
                if self.gesture.result == "→":
                    self.do_next()
                elif self.gesture.result == "←":
                    self.do_prev()
                elif self.gesture.result == "↓→":
                    self.close()
        return True

    def on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
        """
            File Dropped
        """
        drop = data.get_uris()[0]
        if "//" in drop:
            self.set_uri(drop)
        else:
            self.set_uri("file://" + drop)

    def on_key_press(self, widget, event, data=None):
        """
            event.state Mix MOD2_MASK
        """
        if event.state == (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.MOD2_MASK):
            val = self.toolbox.volumebar.get_value()
            if event.keyval == Gdk.KEY_Up:
                if val < 100:
                    self.toolbox.volumebar.set_value(val + 1)
                return True
            elif event.keyval == Gdk.KEY_Down:
                if val > 0:
                    self.toolbox.volumebar.set_value(val - 1)
                return True
        else:
            if event.keyval == Gdk.KEY_Up:
                self.do_prev()
                return True
            elif event.keyval == Gdk.KEY_Down:
                self.do_next()
                return True
            elif event.keyval == Gdk.KEY_Right:
                if self.player.get_state(1)[1] == Gst.State.PAUSED:
                    self.set_next_frame(1.0)
            elif event.keyval == Gdk.KEY_Left:
                if self.player.get_state(1)[1] == Gst.State.PAUSED:
                    self.set_next_frame(-1.0)

    def on_key_release(self, widget, event, data=None):
        self.set_focus(self.video_window)


########################################################################
#
#                       About Signal Handlers
#
########################################################################


    def put_time_status(self, location):
        sec = location // 1000000000
        allsec = self.playinfo.duration // 1000000000
        if allsec >= 3600:
            s = time.strftime("%H:%M:%S", time.gmtime(sec))
            t = time.strftime("%H:%M:%S", time.gmtime(allsec))
        else:
            s = time.strftime("%M:%S", time.gmtime(sec))
            t = time.strftime("%M:%S", time.gmtime(allsec))
        self.statusbar.timelabel.set_text("{0}/{1}".format(s, t))

    def on_timer(self):
        if self.timer_id == -1:
            return
        try:
            self.playinfo.position = self.player.query_position(Gst.Format.TIME)[1]  # 1.0
        except:
            self.playinfo.position = Gst.CLOCK_TIME_NONE
        try:
            # Change SeekBar and Statusbar Time
            if self.playinfo.position != Gst.CLOCK_TIME_NONE:
                self.toolbox.seekadj.set_value(self.playinfo.position)
                self.put_time_status(self.playinfo.position)
            # GetCursolPos
            a, x, y, b = Gdk.Display.get_default().get_pointer()
            # FullScreen at Controler
            if self.fullinfo.toolbox:
                if self.fullinfo.toolbox.y_pos < y:
                    self.fullinfo.toolbox.show_all()
                else:
                    self.fullinfo.toolbox.hide()
            # Cursor Auto Invisible
            if self.mousepoint.x == x and self.mousepoint.y == y:
                if self.invisiblecount < 5:
                    self.invisiblecount += 1
                else:
                    self.video_window.props.window.set_cursor(self.noncursor)
            else:
                self.invisiblecount = 0;
                self.mousepoint.x = x
                self.mousepoint.y = y
                self.video_window.props.window.set_cursor(None)
            # return True is Repeat
            return self.playinfo.is_playing
        except Exception as e:
            self.messagebox(e)

    def on_seek_value_changed(self, widget, data=None):
        if self.timer_id == -1:
            return
        if self.seekinfo.is_seek:
            location = widget.get_value()
            self.set_play_position(location)
            self.put_time_status(location)

    def on_seek_button_press_event(self, widget, event):
        if self.timer_id == -1:
            return
        if event.button < 4:
            if self.player.get_state(1)[1] == Gst.State.PLAYING:
                self.player.set_state(Gst.State.PAUSED)
                self.seekinfo.playing = True
            GLib.source_remove(self.timer_id)
            self.seekinfo.is_seek = True
            self.seekinfo.button = event.button

    def on_seek_button_release_event(self, widget, event):
        self.set_focus(self.video_window)
        if self.timer_id == -1:
            return
        self.seekinfo.is_seek = False
        if self.seekinfo.playing:
            self.seekinfo.playing = False
            self.player.set_state(Gst.State.PLAYING)
        self.timer_id = GLib.timeout_add(200, self.on_timer)

    def set_volume(self, value):
        if self.player.get_volume(GstAudio.StreamVolumeFormat.CUBIC) != value:
            self.player.set_volume(GstAudio.StreamVolumeFormat.CUBIC, value)

    def on_volume_value_changed(self, widget, data=None):
        self.set_volume(widget.get_value() / 100)

    def on_volume_button_release_event(self, widget, event):
        self.set_focus(self.video_window)

    def on_listbox_button_release(self, widget, data=None):
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return
        s = model.get_value(it, 0)
        if s != self.path.name:
            new_uri = GLib.filename_to_uri("{0}/{1}".format(self.path.dirc, s), None)
            self.set_uri(new_uri)
        self.set_focus(self.video_window)

    def on_aframe_draw(self, widget, cr):
        """
            GtkAspectFrame FillRect
        """
        width = widget.get_allocated_width()
        height = widget.get_allocated_height()
        cr.set_source_rgb(0.0, 0.0, 0.0)
        cr.paint()

    def on_keypad(self, accelGroup, window, keyval, modifier):
        """
            processing ten key
        """
        num = keyval - Gdk.KEY_KP_0
        self.headerbar.sizemenu.get_children()[num].activate()

    def on_keydown(self, accelGroup, window, keyval, modifier):
        """
            processing Non Menu Keybord
        """
        if keyval == Gdk.KEY_F2:
            self.rename()
        elif keyval == Gdk.KEY_F5:
            self.path.dirc = ""
            self.listup_change()
        elif keyval == Gdk.KEY_F7:
            self.toolbox.one.clicked()
        elif keyval == Gdk.KEY_F8:
            self.toolbox.all.clicked()
        elif keyval == Gdk.KEY_F9:
            self.toolbox.rdm.clicked()
        elif keyval == Gdk.KEY_F11:
            self.change_fullscreen()
        elif keyval == Gdk.KEY_F12:
            self.on_list_button_clicked(window)
        elif keyval == Gdk.KEY_Escape:
            if self.fullinfo.is_fullscreen:
                self.change_fullscreen()
            elif self.setting.esc_exit:
                self.close()
        elif keyval == Gdk.KEY_Delete:
            if modifier == Gdk.ModifierType.SHIFT_MASK:
                self.trash_prev()
            elif modifier == Gdk.ModifierType.CONTROL_MASK:
                self.trash_next()
            else:
                self.trash()
        elif keyval == Gdk.KEY_KP_Add or keyval == Gdk.KEY_plus:
            if self.settingwin.speed < 11:
                n = self.settingwin.speed + 1
                self.speedbutton.on_menu(None, n)
        elif keyval == Gdk.KEY_KP_Subtract or keyval == Gdk.KEY_minus:
            if self.settingwin.speed > 0:
                n = self.settingwin.speed - 1
                self.speedbutton.on_menu(None, n)
        elif keyval == Gdk.KEY_KP_Enter or keyval == Gdk.KEY_equal:
            if self.settingwin.speed != 5:
                self.speedbutton.on_menu(None, 5)
        elif keyval == Gdk.KEY_Return or keyval == Gdk.KEY_space:
            self.on_play(window)
        elif keyval == Gdk.KEY_BackSpace:
            self.clear()

    def on_seek_scroll_event(self, widget, event):
        """
            Mouse Wheel event
        """
        delta = 0
        if event.direction in [Gdk.ScrollDirection.UP, Gdk.ScrollDirection.RIGHT]:
            delta = 1
        elif event.direction in [Gdk.ScrollDirection.DOWN, Gdk.ScrollDirection.LEFT]:
            delta = -1
        elif event.direction in [Gdk.ScrollDirection.SMOOTH]:
            unused_res, delta_x, delta_y = event.get_scroll_deltas()
            if delta_x:
                delta = math.copysign(1, delta_x)
            elif delta_y:
                delta = math.copysign(1, -delta_y)
        if delta:
            location = widget.get_value() - (delta * 10000000000)
            if location > 0 and location < self.playinfo.duration:
                self.set_play_position(location)
                self.put_time_status(location)


########################################################################
#
#                       GStreamer Event Handlers
#
########################################################################


    def gst_on_message(self, bus, message):
        """
            GStreamer Callback
        """
        t = message.type
        if t == Gst.MessageType.EOS:
            self.clear()
            # Repeat
            if self.settingwin.repeat > 1:
                Gdk.threads_enter()
                self.do_next()
                Gdk.threads_leave()
            elif self.settingwin.repeat == 1:
                self.set_play_position(0)
                self.player.set_state(Gst.State.PLAYING)
        elif t == Gst.MessageType.ERROR:
            self.clear()
            err, debug = message.parse_error()
            self.messagebox(str(err) + "\n" + str(debug))  
            self.toolbox.change_img(0)
        elif t == Gst.MessageType.STATE_CHANGED:
            if Gst.State.PLAYING == message.parse_state_changed()[1]:
                self.toolbox.change_img(0)
                # This is First Play
                if not self.playinfo.is_playing:
                    try:
                        self.playinfo.duration = self.player.query_duration(Gst.Format.TIME)[1]
                    except:
                        self.playinfo.duration = Gst.CLOCK_TIME_NONE
                    if self.playinfo.duration != Gst.CLOCK_TIME_NONE:
                        self.toolbox.seekadj.set_upper(self.playinfo.duration)
                        self.playinfo.is_playing = True
                        # toolbox Buttons Enabled
                        self.toolbox.set_enable_ctrl(True)
                        # Timer on
                        self.timer_id = GLib.timeout_add(200, self.on_timer)
                        # File Names Listup
                        self.listup_change()
                        # Audio
                        asink = self.player.props.audio_sink
                        if asink:
                            # Audio Track
                            audios = []
                            for i in range(self.player.props.n_audio):
                                audios.append("Track #{0}".format(i+1))
                            self.audiobutton.create_menu(audios)
                            n_audio = len(audios) - 1
                            if self.settingwin.n_audio > n_audio:
                                self.settingwin.n_audio = n_audio
                                self.audiobutton.change_label(n_audio)
                            self.player.props.current_audio = self.settingwin.n_audio
                            # Volume
                            self.toolbox.volumebar.props.sensitive = True
                            v = self.toolbox.volumeadj.get_value() / 100.0
                            self.set_volume(v)
                        else:
                            self.toolbox.volumebar.props.sensitive = False
                        # Video vsink == None @ Music File
                        vsink = self.player.props.video_sink
                        if vsink:
                            for pad in vsink.pads:
                                # W Buffer Off
                                self.video_window.set_double_buffered(False)
                                caps = pad.get_current_caps()
                                #print(caps)
                                #caps = pad.get_caps()[0] # 1.1.3
                                success, num, denom = caps.get_structure(0).get_fraction('framerate')
                                success, width = caps.get_structure(0).get_int("width")
                                success, height = caps.get_structure(0).get_int("height")
                                self.playinfo.framerate = num
                                self.playinfo.vsize.width = width
                                self.playinfo.vsize.height = height
                                #
                                #self.change_video_size()
                                aspect_rate = self.playinfo.vsize.width / self.playinfo.vsize.height
                                self.aframe.set(0.5, 0.5, aspect_rate, False)
                                self.change_video_size()
                        self.set_play_position(self.settingwin.speed)
            else:
                self.toolbox.change_img(1)

    def gst_on_sync_message(self, bus, message):
        """
            GStreamer Sync Func
        """
        if message.get_structure().get_name() == "prepare-window-handle":
            imagesink = message.src
            imagesink.props.force_aspect_ratio = False
            imagesink.set_window_handle(self.xid)

########################################################################
#
#                               app
#
########################################################################

class Y901Application(Gtk.Application):
    def __init__(self):
        Gtk.Application.__init__(
                self,
                #application_id="org.sasakima.y901x",
                flags=Gio.ApplicationFlags.HANDLES_OPEN )

    def do_activate(self):
        """
            Non args
        """
        #w = self.get_active_window()
        w = Y901window(self)
        w.present()

    def do_open(self, files, n_file, hint):
        """
            In args
        """
        uri = files[0].get_uri()
        #w = self.get_active_window()
        w = Y901window(self)
        w.set_uri(uri)
        w.present()

    def do_startup (self):
        Gtk.Application.do_startup(self)
        # init
        GObject.threads_init()
        Gtk.init(None)
        Gst.init(None)
        # create a menu
        menu = Gio.Menu()
        menu.append("新しいウィンドウ(_N)", "app.new")
        menu.append("このアプリケーションについて(_A)", "app.about")
        menu.append("終了(_Q)", "app.quit")
        self.set_app_menu(menu)
        # option "new"
        new_action = Gio.SimpleAction.new("new", None)
        new_action.connect("activate", self.new_cb)
        self.add_action(new_action)
        # option "about"
        about_action = Gio.SimpleAction.new("about", None)
        about_action.connect("activate", self.about_cb)
        self.add_action(about_action)
        # option "quit"
        quit_action = Gio.SimpleAction.new("quit", None)
        quit_action.connect("activate", self.quit_cb)
        self.add_action(quit_action)
        # create window
        #Y901window(self)

    def new_cb(self, action, parameter):
        w = Y901window(self)
        w.present()

    def about_cb(self, action, parameter):
        t = self.get_active_window()
        c = "Python {0}\nGTK+ {1}.{2}.{3}".format(
            sys.version,
            Gtk.get_major_version(),
            Gtk.get_minor_version(),
            Gtk.get_micro_version() )
        w = Gtk.AboutDialog()
        w.set_transient_for(t)
        w.set_program_name("Y901x")
        w.set_version(VERSION)
        w.set_copyright("Copyright(C) 2009-2014 sasakima-nao")
        w.set_license("GPL")
        w.set_logo(t.y901icon)
        w.set_comments(c)
        w.set_website("http://palepoli.skr.jp/")
        w.run()
        w.destroy()

    def quit_cb(self, action, parameter):
        self.quit()

########################################################################
#
#                               main
#
########################################################################

def main():
    VERSIONSTR = "Y901x {0}".format(VERSION)
    USAGE = "y901x FILE..."
    parser = optparse.OptionParser(usage=USAGE, version=VERSIONSTR)
    options, args = parser.parse_args()
    app = Y901Application()
    app.run(sys.argv)

if __name__ == "__main__":
    main()
